#!/usr/bin/python

import bzrlib.tests
import os
class ShelfTests(bzrlib.tests.TestCaseInTempDir):
    ORIGINAL = '\n\nhello test world\n\n'
    MODIFIED = '\n\ngoodbye test world\n\n'
    DIFF_HEADER = "=== modified file 'test_file'\n"
    DIFF_1 = """--- test_file\t
+++ test_file\t
@@ -1,4 +1,4 @@
 
 
-hello test world
+goodbye test world
 
"""
    DIFF_2 = """--- test_file\t
+++ test_file\t
@@ -1,4 +1,4 @@
 
 
-goodbye test world
+hello test world
 
"""
    def test_shelf(self):
        from bzrlib.branch import Branch
        b = Branch.initialize('.')

        # Create a test file and commit it
        file('test_file', 'w').write(self.ORIGINAL)
        b.working_tree().add('test_file')
        b.working_tree().commit(message='add test_file')

        # Modify the test file
        file('test_file', 'w').write(self.MODIFIED)

        # Check the diff is right
        self.assertEqual(self.capture('diff', retcode=1),
            self.DIFF_HEADER + self.DIFF_1 + '\n')

        # Shelve the changes
        self.run_bzr('shelve', '--all', retcode=1)

        # Make sure there is no diff anymore
        self.assertEqual(self.capture('diff'), '')

        # Make sure the file is actually back the way it was
        self.assertEqual(file('test_file').read(), self.ORIGINAL)

        # Check the shelf is right
        shelf = file('.bzr-shelf').read()
        self.assertEqual(shelf, self.DIFF_1)

        # Unshelve
        self.run_bzr('unshelve', retcode=1)

        # Check the diff is right again
        self.assertEqual(self.capture('diff', retcode=1),
            self.DIFF_HEADER + self.DIFF_1 + '\n')

        # Make sure the file is back the way it should be
        self.assertEqual(file('test_file').read(), self.MODIFIED)

    def test_shelf_nothing_to_shelve(self):
        import os.path
        from bzrlib.branch import Branch
        b = Branch.initialize('.')

        # Create a test file and commit it
        file('test_file', 'w').write(self.ORIGINAL)
        b.working_tree().add('test_file')
        b.working_tree().commit(message='add test_file')

        # Shelve the changes
        self.run_bzr('shelve', '--all')

        if os.path.exists('.bzr-shelf'):
            self.fail("Shelf exists, but it shouldn't")

    def test_shelf_with_revision(self):
        from bzrlib.branch import Branch
        b = Branch.initialize('.')

        # Create a test file and commit it
        file('test_file', 'w').write(self.ORIGINAL)
        b.working_tree().add('test_file')
        b.working_tree().commit(message='add test_file')

        # Modify the test file and commit it
        file('test_file', 'w').write(self.MODIFIED)
        b.working_tree().commit(message='update test_file')

        # Shelve the changes
        self.run_bzr('shelve', '-r', '1', '--all', retcode=1)

        # Check the diff is right
        self.assertEqual(self.capture('diff', retcode=1),
            self.DIFF_HEADER + self.DIFF_2 + '\n')

        # Make sure the file is the way it should be
        self.assertEqual(file('test_file').read(), self.ORIGINAL)

        # Unshelve
        self.run_bzr('unshelve', retcode=1)

        # Make sure the file is back the way it should be
        self.assertEqual(file('test_file').read(), self.MODIFIED)

    def test_shelf_with_two_revisions(self):
        from bzrlib.branch import Branch
        b = Branch.initialize('.')

        cmd = 'shelve -r 1..2'
        (stdout, stderr) = self.run_bzr_captured(cmd.split(), retcode=None)

        self.assertEqual(stderr.split('\n')[0],
            'bzr: ERROR: shelve only accepts a single revision parameter.')

    def test_shelf_with_whitespace(self):
        """Shows that bzr doesn't handle whitespace well"""
        self.run_bzr('init')
        file('file name', 'wb').write('hello')
        self.run_bzr('add')
        self.run_bzr('commit', '-m', 'added')
        file('file name', 'wb').write('goodbye')
        self.run_bzr('shelve', '--all', 'file name', retcode=1)

    def test_whitespace_branches(self):
        os.mkdir('name with space')
        os.chdir('name with space')
        self.run_bzr('init')
        file('filename', 'wb').write('hello')
        self.run_bzr('add')
        self.run_bzr('commit', '-m', 'added')
        file('filename', 'wb').write('goodbye')
        self.run_bzr('shelve', '--all', 'filename', retcode=1)
        os.chdir('..')

    def test_long_filename(self):
        """Regression test for diffstat with long filenames.

        Create a branch with two files, one of which has a long name. Commit.
        Modify both files. Shelve the file with the short name. If diffstat
        has regressed, it will generate a diffstat of the file with the long
        name, and break.
        """
        self.run_bzr('init')
        filename = 'a' * 80
        file(filename, 'wb').write('hello')
        file('foo', 'wb').write('bar')
        self.run_bzr('add')
        self.run_bzr('commit', '-m', 'added')
        file(filename, 'wb').write('goodbye')
        file('foo', 'wb').write('baz')
        self.run_bzr('shelve', '--all', 'foo', retcode=1)

